local bTracker = _G['bTracker'] or CreateFrame('frame', 'bTracker', UIParent)

local addonName, ns = ...

bTracker.config = false
if not bTracker.frames then bTracker.frames = {} end
bTracker.frames.target = _G['targetAnchor']
bTracker.frames.focus = _G['focusAnchor']

local timeformat = function(s)
	if (s < 3600 and s >= 60) then
		return format('%dm', floor(s / 60 + 1))
	elseif (s < 1) then
		return format('%.1f', s)
	elseif (s < 60) then
		return floor(s+1)
	else
		return format('%dh', floor(s/ 3600 + 1))
	end
end

if not _G["LibStub"] or _G["LibStub"].minor < 2 then
	local LibStub = _G["LibStub"] or {libs = {}, minors = {}, minor = 2}
	function LibStub:NewLibrary(major, minor)
		if self.minors[major] and self.minors[major] >= minor then return nil end
		self.minors[major], self.libs[major] = minor, self.libs[major] or {}
		return self.libs[major], self.minors[major]
	end
	function LibStub:GetLibrary(major, silent) return self.libs[major], self.minors[major] end
	function LibStub:IterateLibraries() return pairs(self.libs) end setmetatable(LibStub, { __call = LibStub.GetLibrary })
end

bTracker.Layout = function()
	for _, anchor in pairs(bTracker.frames) do
		if not bTracker.config then
			table.sort(anchor.frames, function(a,b) return a.bar.duration > b.bar.duration end)
			anchor:SetScript('OnUpdate', bTracker.OnUpdateAnchor)
		else
			table.sort(anchor.frames, function(a,b) return a.bar.id > b.bar.id end)
		end
		local k = 0
		for _, frame in pairs(anchor.frames) do
			if frame:IsShown() or bTracker.config then
				k=k+1
				frame:SetSize(anchor:GetWidth(), anchor:GetHeight())
				frame:SetPoint('BOTTOM', anchor, 'TOP', 0, (k-1)*(anchor:GetHeight()+1))
			end
		end
	end
end

bTracker.OnUpdateAnchor = function(self, elapsed)
	if UnitExists(self.unit) then
		self:Show()
	else
		self:Hide()
	end
end

bTracker.OnUpdateBar = function(self, elapsed)
	if self.tt == nil then self.tt = 0 end
	self.tt = self.tt + elapsed
	self:SetValue((self.expiration - self.tt)/(self.duration/100))
	self.time:SetText(timeformat(self.expiration - self.tt))
	
	if self.expiration - self.tt <= 0 then
		self:SetScript('OnUpdate', nil)
		self.tt = 0
		self:GetParent():Hide()
	end
end

bTracker.UpdateAll = function()
	for _, anchor in pairs(bTracker.frames) do
		for i, frame in pairs(anchor.frames) do
			frame:Hide()
			if bTrackerDB[anchor.unit].hide then break end
			for j=1,40 do
				local name, _, icon, _, _, duration, expirationTime, unitCaster, _, _, spellId = UnitAura(frame.bar.unit, j, "HARMFUL")
				if spellId and duration and frame.bar.id == spellId and unitCaster == 'player' then
					frame.bar:SetMinMaxValues(0, 100)
					frame.bar.duration = duration
					frame.bar.expiration = expirationTime - GetTime()
					frame.bar:SetValue(100)
					
					frame.bar.tt = 0
					frame.bar:SetScript('OnUpdate', bTracker.OnUpdateBar)
					if not bTrackerDB.auras[frame.bar.id].hide then
						frame:Show()
					end
				end
			end
		end
	end
	bTracker.Layout()
end

bTracker.MakeFrame = function(unit, id)
	if _G['b'..unit..id] == nil then
		local frame = CreateFrame('Frame', 'b'..unit..id, UIParent, 'bStatusBarTemplate')
		frame.bar.text:SetText(bTrackerDB.auras[id].text)
		frame.bar.id = id
		frame.bar.expiration = 0
		frame.bar.duration = 0
		frame.bar.spell = bTrackerDB.auras[id].text
		frame.bar.unit = unit
		frame.icon:SetTexture(bTrackerDB.auras[id].icon)
		frame.icon:SetTexCoord(.1,.9,.1,.9)
		tinsert(bTracker.frames[unit].frames, frame)
	end
	bTracker.Layout()
end

bTracker:RegisterEvent('COMBAT_LOG_EVENT_UNFILTERED')
bTracker:RegisterEvent('PLAYER_FOCUS_CHANGED')
bTracker:RegisterEvent('PLAYER_TARGET_CHANGED')
bTracker:RegisterEvent('UNIT_AURA')
bTracker:RegisterEvent('PLAYER_TALENT_UPDATE')
bTracker:RegisterEvent('PLAYER_ENTERING_WORLD')
bTracker:RegisterEvent('ADDON_LOADED')
bTracker:SetScript('OnEvent', function(self, event, arg1, ...)
	if bTracker.config then return end
	
	if (event=='ADDON_LOADED' and arg1 == addonName) or event == 'PLAYER_ENTERING_WORLD' or event=='PLAYER_TALENT_UPDATE' then
		bTrackerDB = _G.bTrackerDB or {}
		if bTrackerDB.auras == nil then bTrackerDB.auras = {} end
		if bTrackerDB.target == nil then bTrackerDB.target = { hide = false } end
		if bTrackerDB.focus == nil then bTrackerDB.focus = { hide = false } end
		
		for _,v in pairs(bTrackerDB.auras) do 
			bTracker.MakeFrame('target', v.spellId) 
			bTracker.MakeFrame('focus', v.spellId) 
		end
	elseif event == 'UNIT_AURA' or event == 'PLAYER_TARGET_CHANGED' or event == 'COMBAT_LOG_EVENT_UNFILTERED' then
		local eventType, _, sourceGUID, sourceName, sourceFlags, sourceFlags2, destGUID, destName, destFlags, destFlags2 = ...
		if event == 'COMBAT_LOG_EVENT_UNFILTERED' then
			if eventType ~= 'SPELL_AURA_APPLIED' and eventType ~= 'SPELL_AURA_REMOVED' and eventType ~= 'SPELL_AURA_REFRESH' and eventType ~= 'SPELL_DISPEL' then return end
		end
		
		if UnitExists('target') then
			for i=1, 40 do
				local name, _, icon, _, _, duration, expirationTime, unitCaster, _, _, spellId = UnitAura('target', i, "HARMFUL")
				if unitCaster == 'player' then
					if bTrackerDB.auras[spellId] == nil then
						bTrackerDB.auras[spellId] = { spellId = spellId, icon = icon, text = name, hide = false }
					end
					if _G['btarget'..spellId] == nil then
						bTracker.MakeFrame('target', spellId)
					end
				end
			end
		end
		
		if UnitExists('focus') then
			for i=1, 40 do
				local name, _, icon, _, _, duration, expirationTime, unitCaster, _, _, spellId = UnitAura('focus', i, "HARMFUL")
				if unitCaster == 'player' then
					if bTrackerDB.auras[spellId] == nil then
						bTrackerDB.auras[spellId] = { spellId = spellId, icon = icon, text = name, hide = false }
					end
					if _G['bfocus'..spellId] == nil then
						bTracker.MakeFrame('focus', spellId)
					end
				end
			end
		end
		bTracker.UpdateAll()
		
	end
end)

bTracker.ConfigTracker = function(isConfig)
	if isConfig == true then
		for _, anchor in pairs(bTracker.frames) do
			anchor:SetScript('OnUpdate', nil)
			anchor:EnableMouse(true)
			anchor:Show()
			anchor:SetScript('OnMouseUp', function(self, buttonkey)
				if buttonkey == 'LeftButton' then
					if bTrackerDB[anchor.unit].hide then
						bTrackerDB[anchor.unit].hide = false
						self:SetAlpha(1)
					else
						bTrackerDB[anchor.unit].hide = true
						self:SetAlpha(.3)
					end
				end
			end)
			if bTrackerDB[anchor.unit].hide then anchor:SetAlpha(.3) else anchor:SetAlpha(1) end
			
			for _, frm in pairs(anchor.frames) do
				frm.bar:SetScript('OnUpdate', nil)
				frm:Show()
				frm.bar:SetValue(100)
				if bTrackerDB.auras[frm.bar.id].hide then frm:SetAlpha(.3) else frm:SetAlpha(1) end
				
				frm.bar:SetScript('OnEnter', function(self) 
					GameTooltip:ClearLines()
					GameTooltip:SetOwner(self, ANCHOR_TOPRIGHT) 
					GameTooltip:AddLine('|cff00ddaashift + click to enable/disable this aura|r')
					GameTooltip:Show() 
				end)
				frm.bar:SetScript('OnLeave', function() GameTooltip:Hide() end)
				
				frm.bar:EnableMouse(true)
				frm.bar:SetScript('OnMouseUp', function(self, buttonkey)
					if buttonkey == 'LeftButton' then
						if bTrackerDB.auras[self.id].hide then
							bTrackerDB.auras[self.id].hide = false
							if _G['btarget'..self.id] then _G['btarget'..self.id]:SetAlpha(1) end
							if _G['bfocus'..self.id] then _G['bfocus'..self.id]:SetAlpha(1) end
						else
							bTrackerDB.auras[self.id].hide = true
							if _G['btarget'..self.id] then _G['btarget'..self.id]:SetAlpha(.3) end
							if _G['bfocus'..self.id] then _G['bfocus'..self.id]:SetAlpha(.3) end
						end
					end
				end)
			end
		end
		bTracker.Layout()
	else
		for _, anchor in pairs(bTracker.frames) do
			anchor:EnableMouse(false)
			anchor:Hide()
			anchor:SetScript('OnUpdate', bTracker.OnUpdateAnchor)
			for _, frm in pairs(anchor.frames) do
				frm.bar:SetScript('OnEnter', nil)
				frm.bar:SetScript('OnLeave', nil)
				frm.bar:SetScript('OnMouseUp', nil)
				if bTrackerDB.auras[frm.bar.id].hide then frm:Hide() else frm:Show() end
				frm.bar:SetScript('OnUpdate', bTracker.OnUpdateBar)
			end
		end
		bTracker.Layout()
	end
end

-- setup slash command
SLASH_Tracker1 = '/btracker';
SLASH_Tracker2 = '/btr';
SlashCmdList['Tracker'] = function(cmd)
	if (cmd:lower() == 'reset') then
		bTrackerDB = {}
		ReloadUI()
	else
		bTracker.config = not bTracker.config
		bTracker.ConfigTracker(bTracker.config)
	end
end